#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <signal.h>

#define INF        999999999999.0

FILE *fp;

double **data;
long long loc1 , loc2;

long long TIMESERIES;
int LENGTH;
double bsf;
double total = 0.0;
    
/* Calculates the distance between two time series x and y. If the distance is
larger than the best so far (bsf) it stops computing and returns the approximate
distance. To get exact distance the bsf argument should be omitted.*/

double distance(double *x, double *y, int length , double best_so_far = INF )
{
    int i;
    double sum = 0;
    double bsf2 = best_so_far*best_so_far;
    for ( i = 0 ; i < length && sum < bsf2  ; i++ )
        sum += (x[i]-y[i])*(x[i]-y[i]);
    return sqrt(sum);
    }

/*Comparison function for qsort function. Compares two time series by using their
distances from the reference time series. */


    
void error(int id)
{
    if(id==1)
        printf("ERROR : Memory can't be allocated!!!\n\n");
        
    exit(1);

    }
    
    
void stop_exec(int sig)
{
    printf("(%lld %lld) is the motif with distance %lf\n\n",loc1,loc2,bsf);
    system("PAUSE");
    exit(1);
    }
    
int main(  int argc , char *argv[] )
{



    double d;
    long long i , j ;
    int abandon = 0 ,  r = 0;
    double ex , ex2 , mean, std;
    long long length;
    int clear = 1;
    double t1,t2;

    /* taking inpput time series from the file in the data array and Normalizing
    them as well. */

    bsf = INF;
    i = 0;
    j = 0;
    ex = ex2 = 0;
    
    signal(SIGINT,stop_exec);
    t1 = clock();

    if(argc < 4 || argc > 4)
    {
        printf("%d",argc);
        exit(1);
    }
    if( argc == 4 )
    {

        //arguments are in the order of filename, number of time series, length of time series, number of reference points
        fp = fopen(argv[1],"r");
        TIMESERIES = atol(argv[2]);
        LENGTH = atoi(argv[3]);
        clear = LENGTH;
  
        data = (double **)malloc(sizeof(double *)*TIMESERIES);
        if( data == NULL  )
            error(1);
        data[0] = (double *)malloc(sizeof(double)*LENGTH);

       // printf("%ld %ld\n\n",data,data[0]);
        if( data[0] == NULL )
            error(1);

        while(fscanf(fp,"%lf",&d) != EOF && i < TIMESERIES)
        {
            data[i][j] = d;
            ex += d;
            ex2 += d*d;
            if( j == LENGTH - 1 )
            {
                mean = ex = ex/LENGTH;
                ex2 = ex2/LENGTH;
                std = sqrt(ex2-ex*ex);
                for( int k = 0 ; k < LENGTH ; k++ )
                    data[i][k] = (data[i][k]-mean)/std;
                ex = ex2 = 0;
                i++;
                if( i != TIMESERIES )
                    data[i] = (double *)malloc(sizeof(double)*LENGTH);
                if( data[i] == NULL )
                    error(1);

                j = 0;
            }
            else
                j++;
        }

        fclose(fp);
    }
 
    for( i = 0 ; i < TIMESERIES; i++ )
        for( j = i+ clear; j < TIMESERIES ; j++ )
        {
            double x = distance(data[i],data[j],LENGTH,bsf);
            total += x;
            if( bsf > x )
            {
                bsf = x;
                loc1 = i;
                loc2 = j;

                }
            }



        /*Printing section of the code.*/

        printf("Motif is the pair ( %lld , %lld ) and the distance is %lf\n",loc1,loc2,bsf);
        t2 = clock(); printf("Execution Time : %lf\n",(t2-t1)/CLOCKS_PER_SEC);

    

}
