#define EXCLENGTH 500


/*Global Variables */
long long TIMESERIES;
long long maxPageSizeTS;
int LENGTH;
int MAXREF;



/* Variables for Input and Output*/

char dirName[100];
FILE * flong;
long long overlap;
double * buffer;

/*Exclusion variables*/

long long clear;
int K;
double X;


/*per block variables*/
double **data1;
double **data2;
long long *ind1;
long long *ind2;
double **Dist1;
double **Dist2;
long long *locations1 , * locations2;
long long pS1, pS2;


/*Global Pointers for sorting and searching */
double **Dist;
double **data;
long long *ind;
long long * locsP;

long long *pageBoundaries;
long long numOfPages;
double * st , * en;

/*Variables Related to Reference Points */
double **dref;
int *rInd;
int iRef;
double *stdRef;
long long *refs;

/*Variables for Output*/
long long loc1 , loc2;
double bsf;
double *ts1 , *ts2;

/*Counters*/

long long countLoads = 0;
long long countDists = 0;
double diskTime;



/* Calculates the distance between two time series x and y. If the distance is
larger than the best so far (bsf) it stops computing and returns the approximate
distance. To get exact distance the bsf argument should be omitted.*/

double distance(double *x, double *y, int length , double best_so_far = INF )
{
    int i;
    double sum = 0;
    double bsf2 = best_so_far*best_so_far;

    countDists++;

    for ( i = 0 ; i < length && sum < bsf2 ; i++ )
        sum += (x[i]-y[i])*(x[i]-y[i]);
    return sqrt(sum);
    }

/*Comparison function for qsort function. Compares two time series by using their
distances from the reference time series. */

int comp1(const void *a,const void *b)
{
    long long *x=(long long *)a;
    long long *y=(long long *)b;

    if (Dist[iRef][*x]>Dist[iRef][*y])
        return 1;
    else if (Dist[iRef][*x]<Dist[iRef][*y])
        return -1;
    else
        return 0;
    }


void error(int id)
{
    if(id==1)
        printf("ERROR : Memory can't be allocated!!!\n\n");
    else if ( id == 2 )
        printf("ERROR : File not Found!!!\n\n");
    else if ( id == 3 )
        printf("ERROR : Can't create Output File!!!\n\n");
    else if ( id == 4 )
        printf("ERROR : Invalid Number of Arguments!!!\n\n");
    else if ( id == 6 )
        printf("ERROR : Can't create page descriptors!!!\n\n");
    else if ( id == 7 )
        printf("ERROR : Page has been corrupted!!!\n\n");
    system("PAUSE");
    exit(1);

    }


long long subLoad(double **data,  long long *ind )
{
    double d;
    long long i = 0 , j  = 0;
    double ex , ex2 , mean, std;
    long long TS = maxPageSizeTS;
    long long h = maxPageSizeTS + LENGTH - 1 - overlap;


    ex = ex2 = 0;

    countLoads++;
    
    
    

    while(  i < maxPageSizeTS + LENGTH - 1)
    {

        if( i < overlap )
            d = buffer[i];
        else
            if(fscanf(flong,"%lf",&d) == EOF)
                break;



        if( i >= h )
            buffer[i-h] = d;



        if( i < maxPageSizeTS )
            ind[i] = i;

        ex += d;
        ex2 += d*d;

        int k = LENGTH-1;
        for(j=((i>=TS)?TS-1:i);j>=((i>=LENGTH)?(i-k):0);j--)
            data[j][i-j] = d;

        if( i >= k )
        {
            mean = ex/LENGTH;
            std = ex2/LENGTH;
            std = sqrt(std-mean*mean);
            ex -= data[i-k][0];
            ex2 -= data[i-k][0]*data[i-k][0];
            for( int u = 0 ; u < LENGTH ; u++ )
               data[i-k][u] = (data[i-k][u]-mean)/std;
        }

        i++;
    }

    if( i >= LENGTH )
        return i-LENGTH+1;
    else
        return 0;


}





void computeDist(double ** data, double ** Dist , long long pS , long long * locations , int flag)
{
    double d;
    long long i , j ;
    int r;
    double ex , ex2 , mean, std;


    i = 0;
    j = 0;
    ex = ex2 = 0;

    for( r = 0 ; r < MAXREF ; r++ )
    {
        if( flag == 0 )
            r = iRef;
            
        for( i = 0 ; i < pS ; i++ )
        {


            if( locations[i] == refs[r] )
            { Dist[r][i] = INF; continue; }
            d = distance(data[i],dref[r],LENGTH);
            Dist[r][i] = d;
            ex += d;
            ex2 += d*d;
            if ( labs(locations[i] - refs[r] ) < clear )  continue;
            if ( d < bsf )  
            {
                 bsf = d; 
                 loc1 = locations[i];
                 loc2 = refs[r];
                 memcpy(ts1,data[i],sizeof(double)*LENGTH);
                 memcpy(ts2,dref[r],sizeof(double)*LENGTH);
            }

        }

        ex = ex/(pS-1);
        ex2 = ex2/(pS-1);
        std = sqrt(ex2-ex*ex);

        rInd[r] = r;
        stdRef[r] = std;
        for ( ; i < maxPageSizeTS ; i++ )
            Dist[r][i] = INF;
            
        if( flag == 0 )
            break;

    }



}



void sortdata(double **data, long long *ind, double ** Dist ,long long pS , long long * locations )
{

    ::Dist = Dist;
    ::data = data;
    ::ind = ind;


    computeDist(data,  Dist , pS , locations , 0);
    qsort(ind,pS,sizeof(long long),comp1);


}




void motif(double **data, double ** Dist ,long long pS , long long * locations )
{


    double d;
    long long i = 0 , j = 0 ;
    long long offset = 0;
    int abandon = 0 ,  r = 0;


    offset = 0;
    abandon = 0;

    while (!abandon && offset < pS)
    {
        abandon = 1;
        offset++;

        for(i = 0 ; i < pS - offset ; i++ )
        {
            long long left = i;
            long long right = i + offset;
          
            double lower_bound = 0;
            
            
            lower_bound = fabs(Dist[iRef][right] - Dist[iRef][left]);
            
            if( lower_bound < bsf )
            {

                abandon = 0;
                if( labs(locations[left]-locations[right]) < clear )
                     continue;

                r = 0;
                do
                {
                    lower_bound = fabs(Dist[r][right] - Dist[r][left]);
                    r++;
                }while( r < MAXREF && lower_bound < bsf );


                if (r >= MAXREF && lower_bound < bsf)
                {

                    d = distance( data[left] , data[right] , LENGTH , bsf );
                    if (d < bsf )
                    {
                        bsf = d;
                        loc1 = locations[left];
                        loc2 = locations[right];
                        memcpy(ts1,data[left],sizeof(double)*LENGTH);
                        memcpy(ts2,data[right],sizeof(double)*LENGTH);


                    }
                }
            }

  
        }//end for i = 0
    }//end while(!abandon)
    
    
//intf("end\n\n");
}


void searchInBlock(double **data, double ** Dist ,long long pS , long long * locations )
{
    double d;
    long long i = 0 , j = 0 ;
    int r = 0;
    long long m = 1;
    long long mid , end;
    long long p;


    while( m < pS )
    {
        p = 1;

        while( p <= pS )
        {
            if( p+m > pS )
                mid = pS+1;
            else
                mid = p+m;

            if( p+2*m > pS )
                end = pS+1;
            else
                end = p+2*m;

            if( mid == end )
                break;

            for( i = p ; i < mid ; i++ )
                for( j = mid ; j < end ; j++ )
                {
 
                    double lower_bound = 0;
                
                if( labs(locations[i]-locations[j]) < clear )
                     continue;

                    r = 0;
                    do
                    {
                        lower_bound = fabs(Dist[r][j] - Dist[r][i]);
                        r++;
                    }while( r < MAXREF && lower_bound < bsf );


                    if (r >= MAXREF && lower_bound < bsf)
                    {

                        d = distance( data[i] , data[j] , LENGTH , bsf );
                        if (d < bsf )
                        {
                            bsf = d;
                            loc1 = locations[i];
                            loc2 = locations[j];
                            memcpy(ts1,data[i],sizeof(double)*LENGTH);
                            memcpy(ts2,data[j],sizeof(double)*LENGTH);


                        }
                    }
                }




            p = p+2*m;
        }
        
        m = 2*m;
    }



}
